@@ -2,7 +2,7 @@
  * This file is part of the bladeRF project:
  *   http://www.github.com/nuand/bladeRF
  *
- * Copyright (C) 2015-2016 Josh Blum
+ * Copyright (C) 2015-2018 Josh Blum
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,11 @@
 #include <cstdio>
 #include <queue>
 
+#if defined(LIBBLADERF_API_VERSION) && (LIBBLADERF_API_VERSION >= 0x02000000)
+#else
+#error "Requires libladerfv2!"
+#endif
+
 /*!
  * Storage for rx commands and tx responses
  */
@@ -61,10 +66,7 @@ public:
         return "bladeRF";
     }
 
-    std::string getHardwareKey(void) const
-    {
-        return "bladeRF";
-    }
+    std::string getHardwareKey(void) const;
 
     SoapySDR::Kwargs getHardwareInfo(void) const;
 
@@ -72,15 +74,9 @@ public:
      * Channels API
      ******************************************************************/
 
-    size_t getNumChannels(const int) const
-    {
-        return 1;
-    }
+    size_t getNumChannels(const int) const;
 
-    bool getFullDuplex(const int, const size_t) const
-    {
-        return true;
-    }
+    bool getFullDuplex(const int, const size_t) const;
 
     /*******************************************************************
      * Stream API
@@ -178,8 +174,12 @@ public:
 
     void setGain(const int direction, const size_t channel, const std::string &name, const double value);
 
+    double getGain(const int direction, const size_t channel) const;
+
     double getGain(const int direction, const size_t channel, const std::string &name) const;
 
+    SoapySDR::Range getGainRange(const int direction, const size_t channel) const;
+
     SoapySDR::Range getGainRange(const int direction, const size_t channel, const std::string &name) const;
 
     /*******************************************************************
@@ -202,13 +202,21 @@ public:
 
     double getSampleRate(const int direction, const size_t channel) const;
 
-    std::vector<double> listSampleRates(const int direction, const size_t channel) const;
+    SoapySDR::RangeList getSampleRateRange(const int direction, const size_t channel) const;
+
+    std::vector<double> listSampleRates(const int direction, const size_t channel) const; //!deprecated
+
+    /*******************************************************************
+     * Bandwidth API
+     ******************************************************************/
 
     void setBandwidth(const int direction, const size_t channel, const double bw);
 
     double getBandwidth(const int direction, const size_t channel) const;
 
-    std::vector<double> listBandwidths(const int direction, const size_t channel) const;
+    SoapySDR::RangeList getBandwidthRange(const int direction, const size_t channel) const;
+
+    std::vector<double> listBandwidths(const int direction, const size_t channel) const; //!deprecated
 
     /*******************************************************************
      * Time API
@@ -221,12 +229,29 @@ public:
     void setHardwareTime(const long long timeNs, const std::string &what = "");
 
     /*******************************************************************
-     * Register API
+     * Sensor API
      ******************************************************************/
 
-    void writeRegister(const unsigned addr, const unsigned value);
+    std::vector<std::string> listSensors(void) const;
 
-    unsigned readRegister(const unsigned addr) const;
+    SoapySDR::ArgInfo getSensorInfo(const std::string &key) const;
+
+    std::string readSensor(const std::string &key) const;
+
+    std::vector<std::string> listSensors(const int direction, const size_t channel) const;
+
+    SoapySDR::ArgInfo getSensorInfo(const int direction, const size_t channel, const std::string &key) const;
+
+    std::string readSensor(const int direction, const size_t channel, const std::string &key) const;
+
+    /*******************************************************************
+     * Register API
+     ******************************************************************/
+    std::vector<std::string> listRegisterInterfaces(void) const;
+
+    void writeRegister(const std::string &name, const unsigned addr, const unsigned value);
+
+    unsigned readRegister(const std::string &name, const unsigned addr) const;
 
     /*******************************************************************
      * Settings API
@@ -258,9 +283,9 @@ public:
 
 private:
 
-    static bladerf_module _dir2mod(const int direction)
+    static bladerf_channel _toch(const int direction, const size_t channel)
     {
-        return (direction == SOAPY_SDR_RX)?BLADERF_MODULE_RX:BLADERF_MODULE_TX;
+        return (direction == SOAPY_SDR_RX)?BLADERF_CHANNEL_RX(channel):BLADERF_CHANNEL_TX(channel);
     }
 
     static std::string _err2str(const int err)
@@ -315,6 +340,8 @@ private:
         _rxMinTimeoutMs = long((2*1000*_rxBuffSize)/_rxSampRate);
     }
 
+    bool _isBladeRF1;
+    bool _isBladeRF2;
     double _rxSampRate;
     double _txSampRate;
     bool _inTxBurst;
@@ -328,6 +355,8 @@ private:
     int16_t *_txConvBuff;
     size_t _rxBuffSize;
     size_t _txBuffSize;
+    std::vector<size_t> _rxChans;
+    std::vector<size_t> _txChans;
     long _rxMinTimeoutMs;
     std::queue<StreamMetadata> _rxCmds;
     std::queue<StreamMetadata> _txResps;
